import java.util.ArrayList;
import java.util.Arrays;


public class Grupa implements Comparable<Grupa> {
	
	/** Grupa predmeta koji se režu od odabranog materijala */
	private ArrayList<Integer> group;
	/** Duljina odabranog materijala */
	private int stockLength;
	/** Ostatak nakon rezanja grupe predmeta iz materijala */
	private int remainder;
	
	/**
	 * Konstruktor objekta
	 * 
	 * @param group grupa predmeta koji će se rezati od jednog komada materijala
	 * @param stockLength duljina komada materijala od kojeg će se grupa rezati
	 * @param remainder	neiskorišteni dio materijala nakon rezanja
	 */
	public Grupa(ArrayList<Integer> group, int stockLength, int remainder) {
		super();
		this.group = group;
		this.stockLength = stockLength;
		this.remainder = remainder;
	}

	/**
	 * Vraća grupu predmeta koji se režu od odabranog materijala
	 * 
	 * @return grupa predmeta
	 */
	public ArrayList<Integer> getGroup() {
		return group;
	}

	/**
	 * Vraća duljinu odabranog materijala
	 * 
	 * @return duljina materijala
	 */
	public int getStockLength() {
		return stockLength;
	}

	/**
	 * Vraća duljinu materijala koja ostaje nakon rezanja grupe predmeta iz materijala
	 * 
	 * @return ostatak nakon rezanja
	 */
	public int getRemainder() {
		return remainder;
	}

	/* (non-Javadoc)
	 * @see java.lang.Comparable#compareTo(java.lang.Object)
	 */
	public int compareTo(Grupa grupa) {
		if (this.remainder < grupa.remainder)
			return -1;
		else if (this.remainder > grupa.remainder)
			return 1;
		else
			return 0;
		
	}
	
	@Override
	public String toString(){
		Integer[] grupa = group.toArray(new Integer[0]);
		Arrays.sort(grupa);
		String ispis = String.valueOf(stockLength) + '\t' + String.valueOf(grupa[0]);
		for (int i = 1 ; i < grupa.length ; i++ ){
			ispis += " + " + String.valueOf(grupa[i]);
		}
		ispis +=  "\t\t" + String.valueOf(remainder);
		return ispis;
	}

}
