import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.geom.Line2D;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;


public class Grupa implements Comparable<Grupa> {
	
	/** Predmeti i točke na materijalu u koje ih se smješta prilikom rezanja */
	private HashMap<InsertPoint, Predmet> group;
	/** Predmeti koji se režu od odabranog materijala i njihov broj */
	private HashMap<Integer, Integer> predmeti;
	/** Lista mogućih točaka smještanja idućeg materijala */
	private ArrayList<InsertPoint> insertPoints;
	/** Ostatak nakon rezanja grupe predmeta iz materijala */
	private int remainder;
	/** Visina materijala */
	private int stockHeight;
	/** Širina materijala */
	private int stockWidth;
	
	/**
	 * Konstruktor objekta
	 * 
	 * @param stockWidth širina materijala
	 * @param stockHeight visina materijala
	 */
	public Grupa(int stockWidth, int stockHeight) {
		super();
		this.group = new HashMap<InsertPoint, Predmet>();
		this.stockWidth = stockWidth;
		this.stockHeight = stockHeight;
		this.remainder = stockHeight*stockWidth;
		this.predmeti = new HashMap<Integer, Integer>();
		this.insertPoints = new ArrayList<InsertPoint>();
		insertPoints.add(new InsertPoint(0, 0));
	}
	
	/**
	 * Pokušava dodati predmet u grupu
	 * 
	 * @param predmet predmet koji se pokušava dodati u grupu
	 * @return true ako je dodavanje uspjelo, inače false
	 */
	public boolean addItem(Predmet predmet){
		InsertPoint[] tocke = insertPoints.toArray(new InsertPoint[0]);
		Arrays.sort(tocke);
		for(InsertPoint i : tocke){
			int x1 = i.getX();
			int x2 = i.getX() + predmet.getSirina();
			int y1 = i.getY();
			int y2 = i.getY() + predmet.getVisina();
			if (x2 > stockWidth || y2 > stockHeight){
				continue;
			}
			boolean fits = true;
			for(InsertPoint j : group.keySet()){
				if (j.equals(i)){
					fits = false;
					break;
				}
				int tempX = j.getX() + group.get(j).getSirina();
				int tempY = j.getY() + group.get(j).getVisina();
				if (x1 >= j.getX() && y1 >= j.getY() && x2 <= tempX && y2 <= tempY){
					fits = false;//unutar nekog je
					break;
				}
				if ((x1 > j.getX() && y1 > j.getY() && x1 < tempX && y1 < tempY) || (x1 > j.getX() && y2 > j.getY() && x1 < tempX && y2 < tempY) || (x2 > j.getX() && y1 > j.getY() && x2 < tempX && y1 < tempY) || (x2 > j.getX() && y2 > j.getY() && x2 < tempX && y2 < tempY)){
					fits = false;//jedan vrh je unutar drugog kvadrata
					break;
				}
				if ((x1 < j.getX() && y1 < j.getY() && x2 > j.getX() && y2 > j.getY()) || (x1 < tempX && y1 < j.getY() && x2 > tempX && y2 > j.getY()) || (x1 < j.getX() && y1 < tempY && x2 > j.getX() && y2 > tempY) || (x1 < tempX && y1 < tempY && x2 > tempX && y2 > tempY)){
					fits = false;//vrh drugog kvadrata je unutar ovog
					break;
				}
				if (!(x2 <= j.getX() || x1 >= tempX || y2 <= j.getY() || y1 >= tempY)){
					fits = false;//nisu sve tocke sa jedne strane nekog kvadrata
					break;
				}
			}
			if (!fits)
				continue;
			add(predmet, i);
			return true;
		}
		return false;
	}
	
	/**
	 * Izvršava dodavanje predmeta u zapis grupe
	 * 
	 * @param predmet predmet koji se dodaje
	 * @param ip točka u koju se predmet smješta
	 */
	private void add(Predmet predmet, InsertPoint ip){
		InsertPoint prvaTocka = new InsertPoint((ip.getX()+predmet.getSirina()), ip.getY());
		InsertPoint drugaTocka = new InsertPoint(ip.getX(), (ip.getY()+predmet.getVisina()));
		insertPoints.add(prvaTocka);
		insertPoints.add(drugaTocka);
		insertPoints.remove(ip);
		group.put(ip, predmet);
		if (predmeti.containsKey(predmet.getID()))
			predmeti.put(predmet.getID(), predmeti.get(predmet.getID())+1);
		else
			predmeti.put(predmet.getID(), 1);
		remainder = remainder - (predmet.getSirina()*predmet.getVisina());
	}

	/**
	 * Vraća grupu predmeta koji se režu od odabranog materijala
	 * 
	 * @return grupa predmeta
	 */
	public HashMap<Integer, Integer> getPredmeti() {
		return predmeti;
	}

	/**
	 * Vraća duljinu materijala koja ostaje nakon rezanja grupe predmeta iz materijala
	 * 
	 * @return ostatak nakon rezanja
	 */
	public int getRemainder() {
		return remainder;
	}

	/* (non-Javadoc)
	 * @see java.lang.Comparable#compareTo(java.lang.Object)
	 */
	public int compareTo(Grupa grupa) {
		if (this.remainder < grupa.remainder)
			return -1;
		else if (this.remainder > grupa.remainder)
			return 1;
		else
			return 0;
		
	}
	
	@Override
	public String toString(){
		String izlaz = new String();
		for(InsertPoint i : group.keySet()){
    		izlaz += "(" + i.getX() + "," + i.getY() + ") - (" + (i.getX()+group.get(i).getSirina()) + "," + (i.getY()+group.get(i).getVisina()) + ")\n";
    	}
		return izlaz;
	}
	
	/**
	 * Stvara sliku rješenja
	 * 
	 * @return slika rješenja
	 */
	public BufferedImage toPicture(){
		int sirinaMjerila = 35;
		BufferedImage image = new BufferedImage(stockWidth + sirinaMjerila + 15, stockHeight + sirinaMjerila + 10, BufferedImage.TYPE_BYTE_INDEXED);
    	Graphics2D graphics = image.createGraphics();
    	graphics.setBackground(Color.white);
    	graphics.clearRect(0, 0, stockWidth + sirinaMjerila + 15, stockHeight + sirinaMjerila + 10);
    	
    	graphics.setColor(Color.black);
    	int korakMjerila = 100;
    	int brojac = 0;
    	for(double x = sirinaMjerila ; (x - 0.1) <= (stockWidth + sirinaMjerila) ; x += korakMjerila){
    		if (brojac % 5 == 0) {
    			graphics.draw(new Line2D.Double(x, sirinaMjerila, x, sirinaMjerila - 10));
    			
    		} else {
    			graphics.draw(new Line2D.Double(x, sirinaMjerila, x, sirinaMjerila - 5));
    		}
    		graphics.drawString(String.valueOf(brojac * korakMjerila), (float) (x - 15), sirinaMjerila - 15);
    		brojac++;
    	}
    	brojac = 0;
    	for(double y = sirinaMjerila ; (y - 0.1) <= (stockHeight + sirinaMjerila) ; y += korakMjerila){
    		if (brojac % 5 == 0) {
    			graphics.draw(new Line2D.Double(sirinaMjerila, y, sirinaMjerila - 10, y));
    			
    		} else {
    			graphics.draw(new Line2D.Double(sirinaMjerila, y, sirinaMjerila - 5, y));
    		}
    		graphics.drawString(String.valueOf(brojac * korakMjerila), (float) (sirinaMjerila - 35), (float) (y + 5));
    		brojac++;
    	}
    	
    	Rectangle2D.Double rect;
    	for(InsertPoint i : group.keySet()){
    		rect = new Rectangle2D.Double(i.getX() + sirinaMjerila, stockHeight - i.getY() - group.get(i).getVisina() + sirinaMjerila, group.get(i).getSirina(), group.get(i).getVisina());
    		graphics.setColor(Color.cyan);
			graphics.fill(rect);
			graphics.setColor(Color.blue);
			graphics.draw(rect);
    	}
    	graphics.dispose();
		return image;
	}

}
