import java.util.ArrayList;
import java.util.Arrays;


public class HGrupa implements Comparable<HGrupa> {
	
	/** Grupa predmeta koji se režu od odabranog materijala */
	private ArrayList<Integer> group;
	/** Duljina odabranog materijala */
	private int stockLength;
	/** Ostatak nakon rezanja grupe predmeta iz materijala */
	private int remainder;
	
	
	/**
	 * Konstruktor objekta
	 * 
	 * @param group grupa predmeta koji će se rezati od jednog komada materijala
	 * @param stockLength duljina komada materijala od kojeg će se grupa rezati
	 * @param remainder	neiskorišteni dio materijala nakon rezanja
	 */
	public HGrupa(ArrayList<Integer> group, int stockLength, int remainder) {
		super();
		this.group = group;
		this.stockLength = stockLength;
		this.remainder = remainder;
	}


	/**
	 * Vraća grupu predmeta koji se režu od odabranog materijala
	 * 
	 * @return grupa predmeta
	 */
	public ArrayList<Integer> getGroup() {
		return group;
	}
	
	/**
	 * Briše predmet iz grupe
	 * 
	 * @param element duljina predmeta koji se briše iz grupe
	 * @param debljinaReza debljina rezne ploče
	 */
	public void removeFromGroup(Integer element, int debljinaReza){
		this.group.remove(element);
		this.remainder += element + debljinaReza;
	}
	
	/**
	 * Dodaje predmet u grupu
	 * 
	 * @param element duljina predmeta koji se briše iz grupe
	 * @param debljinaReza debljina rezne ploče
	 */
	public void addToGroup(Integer element, int debljinaReza){
		this.group.add(element);
		this.remainder -= (element + debljinaReza);
		if (this.remainder < 0)
			this.remainder = 0;
	}

	/**
	 * Vraća duljinu odabranog materijala
	 * 
	 * @return duljina materijala
	 */
	public int getStockLength() {
		return stockLength;
	}


	/**
	 * Vraća duljinu materijala koja ostaje nakon rezanja grupe predmeta iz materijala
	 * 
	 * @return ostatak nakon rezanja
	 */
	public int getRemainder() {
		return remainder;
	}


	/* (non-Javadoc)
	 * @see java.lang.Comparable#compareTo(java.lang.Object)
	 */
	public int compareTo(HGrupa grupa) {
		if (this.remainder < grupa.remainder)
			return -1;
		else if (this.remainder > grupa.remainder)
			return 1;
		else
			return 0;
		
	}
	
	@Override
	public String toString(){
		String ispis = String.valueOf(stockLength) + '\t' + String.valueOf(group.get(0));
		for (int i = 1 ; i < group.size() ; i++ ){
			ispis += " + " + String.valueOf(group.get(i));
		}
		ispis +=  "\t\t" + String.valueOf(remainder);
		return ispis;
	}
}
